/*
dothis.c, Copyright Toby Jaffey, April 28th 1999
Distributed under the GNU GPL
bug reports and ideas to psystrj@granby.nottingham.ac.uk

Thu Apr 29 15:50:59 BST 1999
Added routines to save the window position in a file
*/

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define RCFILE ".dothisrc"

void handle_buttons(GtkWidget *w, gpointer d);
GtkWidget *addButton(GtkWidget *vbox,GtkWidget *window,char *label,int show,GtkSignalFunc func,gpointer data);
GtkWidget *createDrawWindow(int xsize,int ysize,int xoff,int yoff,char *title);
GtkWidget *createHBox(GtkWidget *window);
GtkWidget *createVBox(GtkWidget *window);
void quit(GtkWidget *w);
GtkWidget *addText(GtkWidget *box,GtkWidget *window,gchar *starttext,GtkSignalFunc func);
void text_done(GtkWidget *w,GtkWidget *e);
void savePos(char *name,int x,int y);
void getPos(char *name,int *x,int *y);

char *options=NULL,*command;
GtkWidget *entry,*mainwindow;


int main(int argc, char *argv[])
{
GtkWidget *hbox,*hbox2,*vbox;
int x,y;

if (argc!=4)
	{
	fprintf(stderr,"\nNot enough args\nUsage: %s title command starttext\neg. %s xtermify \"xterm -e\" defaulttext\n\n",argv[0],argv[0]);
	exit(1);
	}
command = argv[2];

getPos(RCFILE,&x,&y);

gtk_init(&argc, &argv);

mainwindow = createDrawWindow(0,0,x,y,argv[1]);
vbox = createVBox(mainwindow);
hbox = createHBox(vbox);
hbox2 = createHBox(vbox);

entry = addText(hbox,mainwindow,argv[3],GTK_SIGNAL_FUNC(text_done));
addButton(hbox2,mainwindow,"OK",1,GTK_SIGNAL_FUNC(handle_buttons),(gpointer)1);
addButton(hbox2,mainwindow,"Cancel",1,GTK_SIGNAL_FUNC(handle_buttons),(gpointer)0);
gtk_entry_select_region(GTK_ENTRY(entry),0,strlen(argv[2]));
gtk_widget_show(mainwindow);
gtk_widget_grab_focus(entry);

gtk_main();

return 0;
} // end main

void handle_buttons(GtkWidget *w,gpointer d)
{
char *s;
int x,y;

s = malloc(1024);
if (s==NULL)
	{
	fprintf(stderr,"\nOut of mem!\n\n");
	free(s);
	exit(2);
	}

switch((int)d)
	{
	case 0:
	quit(mainwindow);
	break;
	
	case 1:
	if (options==NULL)
	text_done(mainwindow,entry);
	gdk_window_get_root_origin(mainwindow->window,&x,&y);
	savePos(RCFILE,x,y);
	sprintf(s,"%s %s&",command,options);
	system(s);
	free(s);
	quit(mainwindow);
	break;
	}
} // end handle button presses


GtkWidget *addButton(GtkWidget *vbox,GtkWidget *window,char *label,int show,GtkSignalFunc func,gpointer data)
{
GtkWidget *button;
button = gtk_button_new_with_label(label);
gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(func),data);
if (show) gtk_widget_show(button);
return button;
} // end add a button to the vbox


GtkWidget *createDrawWindow(int xsize,int ysize,int xoff,int yoff,char *title)
{
GtkWidget *window;  
window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
gtk_window_set_policy((GtkWindow *)window,0,0,1);
gtk_widget_set_uposition(window,xoff,yoff);
gtk_window_set_title((GtkWindow *)window,title);
gtk_signal_connect(GTK_OBJECT(window),"destroy",GTK_SIGNAL_FUNC (quit), NULL);
return window;
} // end create window to draw in.


GtkWidget *createHBox(GtkWidget *window)
{
GtkWidget *hbox;
hbox = gtk_hbox_new(FALSE, 1);
gtk_container_add(GTK_CONTAINER(window),hbox);
gtk_widget_show(hbox);
return hbox;
} // end create hbox for packing


GtkWidget *createVBox(GtkWidget *window)
{
GtkWidget *hbox;
hbox = gtk_vbox_new(FALSE, 1);
gtk_container_add(GTK_CONTAINER(window),hbox);
gtk_widget_show(hbox);
return hbox;
} // end create hbox for packing


void quit(GtkWidget *w)
{
gtk_widget_hide(w);
gtk_exit(0);
} // end prog


GtkWidget *addText(GtkWidget *box,GtkWidget *window,gchar *starttext,GtkSignalFunc func)
{
GtkWidget *text;
text = gtk_entry_new_with_max_length(255);
gtk_entry_set_text(GTK_ENTRY(text),starttext);
gtk_box_pack_start(GTK_BOX(box),text,TRUE,TRUE,0);
gtk_signal_connect(GTK_OBJECT(text),"activate",GTK_SIGNAL_FUNC(func),text);
gtk_widget_show(text);
return text;
} // end add a button to the vbox


void text_done(GtkWidget *w,GtkWidget *e)
{
options = gtk_entry_get_text(GTK_ENTRY(e));
handle_buttons(mainwindow,(gpointer)1);
}


void savePos(char *name,int x,int y)
{
FILE *fp;
char *filename;

filename = malloc(1024);
if (filename==NULL)	
	{
	printf("Out of mem!\n\n");
	exit(1);
	}

strcpy(filename,getenv("HOME"));
strcat(filename,"/");
strcat(filename,name);

//fprintf(stderr,"-%s-",filename);

fp = fopen(filename,"w");
if (fp==NULL)
	{
	fprintf(stderr,"\nCould not create file %s\n",filename);
	free(filename);
	return;
	}
else
	{
	fprintf(fp,"%d %d\n",x,y);
	fclose(fp);
	free(filename);
	}
}



void getPos(char *name,int *x,int *y)
{
FILE *fp;
char *filename;

filename = malloc(1024);
if (filename==NULL)	
	{
	printf("Out of mem!\n\n");
	exit(1);
	}

strcpy(filename,getenv("HOME"));
strcat(filename,"/");
strcat(filename,name);

fp = fopen(filename,"r");
if (fp==NULL)
	{
	fprintf(stderr,"\nCould not open file %s\n",filename);
	fprintf(stderr,"\nCreating config file %s\n",filename);
	savePos(name,10,10);
	free(filename);
	return;
	}
else
	{
	if (fscanf(fp,"%d %d\n",x,y)!=2)
		{
		fprintf(stderr,"%s contains invalid geometry data\n",filename);
		}
	fclose(fp);
	free(filename);
	}
}


